"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const execa = require("execa");
exports.execa = execa;
const pathLib = require("path");
const utils_1 = require("./utils");
const fs_1 = require("./fs");
const stream_1 = require("stream");
function exec(commands, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (utils_1.Is.str(commands)) {
            return execa.shell(commands, options);
        }
        let rets = [];
        for (let cmd of commands) {
            rets.push(yield execa.shell(cmd, options));
        }
        return rets;
    });
}
exports.exec = exec;
exports.spawn = execa;
class ShellContext {
    constructor() {
        this._cwdStack = [process.cwd()];
        this._env = {};
        this.logCommand = false;
        this.sleep = utils_1.sleep;
    }
    get cwd() {
        return this._cwdStack[this._cwdStack.length - 1];
    }
    get _logger() {
        let logger = require('./logger').logger;
        return logger;
    }
    cd(dir) {
        this._cwdStack[this._cwdStack.length - 1] = pathLib.resolve(this._cwdStack[this._cwdStack.length - 1], dir);
        return this;
    }
    pushd(dir) {
        this._cwdStack.push(pathLib.resolve(this._cwdStack[this._cwdStack.length - 1], dir));
        return this;
    }
    popd() {
        this._cwdStack.pop();
        return this;
    }
    exec(commands, options) {
        this._logCmd(commands);
        let p = exec(commands, Object.assign({ cwd: this.cwd, env: Object.assign({}, process.env, this._env), stdio: this.redirectLog ? 'pipe' : 'inherit' }, options));
        let redirectLogStream = this._getRedirectLogFile();
        if (redirectLogStream) {
            // tslint:disable-next-line:no-floating-promises
            p.then(p => {
                p.stdout && redirectLogStream.write(p.stdout);
                p.stderr && redirectLogStream.write(p.stderr);
                redirectLogStream.end();
            });
        }
        // tslint:disable-next-line:no-floating-promises
        p.catch(err => {
            this._logger.error('Exec failed: ', commands);
            if (redirectLogStream) {
                redirectLogStream.end();
            }
            throw err;
        });
        return p;
    }
    spawn(file, args = [], options) {
        const command = file + ' ' + args.map(a => `"${a.replace(/"/g, '\\"')}"`).join(' ');
        this._logCmd(command);
        let p = exports.spawn(file, args, Object.assign({ cwd: this.cwd, env: Object.assign({}, process.env, this._env), stdio: this.redirectLog ? 'pipe' : 'inherit' }, options));
        let redirectLogStream = this._getRedirectLogFile();
        if (redirectLogStream) {
            // tslint:disable-next-line:no-floating-promises
            p.then(p => {
                p.stdout && redirectLogStream.write(p.stdout);
                p.stderr && redirectLogStream.write(p.stderr);
                redirectLogStream.end();
            });
        }
        // tslint:disable-next-line:no-floating-promises
        p.catch(err => {
            this._logger.error('Exec failed: ', command);
            if (redirectLogStream) {
                redirectLogStream.end();
            }
            throw err;
        });
        return p;
    }
    env(key, val) {
        if (arguments.length === 1) {
            return this._env[key];
        }
        if (utils_1.Is.defed(val)) {
            this._env[key] = val;
        }
        else {
            delete this._env[key];
        }
        return this;
    }
    resetEnv() {
        this._env = {};
        return this;
    }
    _getRedirectLogFile() {
        if (!this.redirectLog)
            return null;
        if (this.redirectLog instanceof stream_1.Writable) {
            return this.redirectLog;
        }
        let logFile = utils_1.Is.str(this.redirectLog) ? this.redirectLog : utils_1.DefaultLogFile;
        return fs_1.fs.createWriteStream(logFile, { mode: fs_1.fs.constants.O_APPEND });
    }
    _logCmd(cmd) {
        if (this.logCommand) {
            let env = Object.keys(this._env).map(k => `${k}=${this._env[k] || ''}`).join(' ');
            if (env) {
                env += ' ';
            }
            cmd = Array.isArray(cmd) ? cmd : [cmd];
            cmd.forEach(cmd => {
                this._logger.info(`$ ${env}${cmd}`);
            });
        }
    }
}
exports.ShellContext = ShellContext;
function shell(callback) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return callback(new ShellContext());
    });
}
exports.shell = shell;
//# sourceMappingURL=exec.js.map